%evalTr.m Matlab Script - Evaluation of tracking results in the GRAM-RTM
%    Copyright (C) 2013  Ricardo Guerrero-Gómez-Olmedo and Roberto
%    López-Sastre
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

clear all;
clc;

addpath('auxiliary_functions');

%classes
cls = {'car', 'truck', 'van', 'big-truck'};

%If draw is set to true, the code displays the results graphically
draw = true;

%Update with the corresponding path

%Secuence we are going to evaluate. RTMopts.videoName must be: "M-30", "M-30-HD" and "Urban1"
RTMopts.videoName = 'M-30-HD';
%Threshold value
RTMopts.minoverlap = 0.5;
%If the pose is consider during the tracking. RTMopts.videoName could be: ""Pose" or "No Pose"
RTMopts.mode = 'Pose';
%Secuence xml
RTMopts.annopath = ['../Annotations/' RTMopts.videoName '/xml/'];
%Tracking results
RTMopts.tracking = ['../Results/Tracking/' RTMopts.videoName '/Pose/tracking_results.txt'];
%ROI
RTMopts.roi = ['../ROI/' RTMopts.videoName '-map.jpg'];

%Loading ROI
roi = imread(RTMopts.roi);

num_xml = size(dir(RTMopts.annopath),1)-2;

%Loading annotations
gtids=1:num_xml;

gtids_aux=[];
disp('Loading annotations ... (this may take a while)');
parfor i=1:length(gtids)
    recs(i)=PASreadrecord(sprintf(strcat(RTMopts.annopath, '%i.xml'), gtids(i)-1));
end

        
%Giving format to annotations
npos=0;
gt(length(gtids))=struct('BB',[],'ID',[], 'det', []);


for i=1:length(gtids)
    % extract objects of class
    clsinds = [];
    
    if size(recs(i).objects(:),1) == 0
        clsinds=[];
    else
        
        for a=1:size(cls,2)
            clsinds= [clsinds; strmatch(cls(a),{recs(i).objects(:).class},'exact') ];
        end
    end
    
    if size(clsinds,1) ~= 0
        gt(i).BB=cat(1,recs(i).objects(clsinds).bndbox)';
        
        for a=1:size(clsinds)
            b(a) = str2num(recs(i).objects(clsinds(a)).ID);
        end
        
        gt(i).ID = b;
        gt(i).det=false(length(clsinds),1);
        npos=npos+size(clsinds,1);
    else
        gt(i).BB = [];
        gt(i).ID = [];
        gt(i).det = false;
    end
end
disp('Finish loading xml');

disp ('Loading tracking results');

%x1=b1 y1=b2 x2=b3 y2=b4
%ids is the number of the frame where the vehicle was detected
[ids, b1, b2, b3, b4, confidence, ID]= textread(RTMopts.tracking,'%f %f %f %f %f %f %d'); 

counted_cars = max(ID);
BB=[b1 b2 b3 b4]';


% sort detections by decreasing confidence
[sc,si]=sort(-confidence);
ids=ids(si);
BB=BB(:,si);

% assign detections to ground truth objects
nd=length(confidence);

% True positives
tp=zeros(nd,1);
% False positives
fp=zeros(nd,1);
tic;
det_ID = [];
ground_ID = [];
ident_gt = [];
disp ('Performing evaluation ...');
tic
for d=1:nd
    % display progress
    if toc>1
        fprintf('compute: %d/%d\n',d,nd);
        drawnow;
        tic;
    end



    % find ground truth image
     i = ids(d);

    % assign detection to ground truth object if any
    bb=BB(:,d);
    ovmax=-inf;
    

    if i~=0
        final = size(gt(i).BB,2);
    else
        final = 0;
    end

    for j=1:final
        bbgt = [gt(i).BB(j).xmin, gt(i).BB(j).ymin, gt(i).BB(j).xmax, gt(i).BB(j).ymax];
        bi=[max(bb(1),bbgt(1)) ; max(bb(2),bbgt(2)) ; min(bb(3),bbgt(3)) ; min(bb(4),bbgt(4))];
        iw=bi(3)-bi(1)+1;
        ih=bi(4)-bi(2)+1;

        if iw>0 & ih>0                
            % compute overlap as area of intersection / area of union
            ua=(bb(3)-bb(1)+1)*(bb(4)-bb(2)+1)+...
               (bbgt(3)-bbgt(1)+1)*(bbgt(4)-bbgt(2)+1)-...
               iw*ih;
            ov=iw*ih/ua;
            if ov>ovmax
                ovmax=ov;
                jmax=j;
            end
        end
    end

    % assign detection as true positive/don't care/false positive
    if ovmax>=RTMopts.minoverlap
        % Not detected yet
        if ~gt(i).det(jmax)
            tp(d)=1;            % true positive
            gt(i).det(jmax)=true;
        else
            if inROI(roi, bb)
            		fp(d)=1;            % false positive (multiple detection)
            end
        end% Not detected yet
    else
		if inROI(roi, bb)
            		fp(d)=1;            % false positive
		end
    end
end


% compute precision/recall
fp=cumsum(fp);
tp=cumsum(tp);
rec=tp/npos;
prec=tp./(fp+tp);

ap=VOCap(rec,prec);
if draw
    figure(1);
    plot(rec,prec,'-', 'LineWidth', 3);
    xlim([0 1]);
    ylim([0 1]);    
    grid;
    xlabel 'recall'
    ylabel 'precision'
    title(sprintf('subset: %s, AP = %.3f',RTMopts.videoName,ap));
    
    disp (['Counted car; ' num2str(counted_cars)]);
end


save(['tr ' RTMopts.mode ' ' RTMopts.videoName], 'rec', 'prec', 'ap', 'counted_cars');


